/* -------------------------------------------------------
                        ds18b20_demo.c

     Demoprogramm fuer DS18B20 Temperatursensor

     MCU   :  CH32V003

     one_wire_Anschluss DS18B20 auf PA1 (aenderbar)

     14.06.2025  R. Seelig
   ------------------------------------------------------ */
/*

                  CH32V003 A4M6                                        CH32V003F4P6

                  +-----------+                                      +--------------+
        PC1 / sda |1    C   16| PC0                         PD4 / a7 | 1         20 | PD3 / a4
        PC2 / scl |2    H   15| Vdd                   PD5 / a5 / utx | 2    C    19 | PD2 / a3
              PC3 |3    3   14| gnd                   PD6 / a6 / urx | 3    H    18 | PD1 / swio
              PC4 |4    2   13| PA2 / osc0 / a0           nrst / PD7 | 4    3    17 | PC7 / miso
       PC6 / mosi |5    V   12| PA1 / osc1 / a1      PA1 / osc0 / a0 | 5    2    16 | PC6 / mosi
       PC7 / miso |6    0   11| nrst / PD7           PA2 / osc1 / a1 | 6    V    15 | PC5 / sck / scl
       PD1 / swio |7    0   10| PD6 / a6 / urx                   gnd | 7    0    14 | PC4 / a2
         PD4 / a7 |8    3    9| PD5 / a5 / utx                   PD0 | 8    0    13 | PC3
                  +-----------+                                  Vdd | 9    3    12 | PC2 / scl
                                                                 PC0 | 10        11 | PC1 / sda
                                                                     +--------------+
*/

/*
   DS18B20
  +-------+
  |       |
  |       |
  |       |
  +-------+
   |  |  |
   |  |  |
   |  |  |

   G  D  V
   N  A  c
   D  T  c
*/

#include "ch32fun.h"
#include "ch32v003_gpio.h"
#include "uart.h"
#include "my_printf.h"
#include "ds18b20_single.h"

#define printf       my_printf

extern const uint8_t bmps7asc [96];     // Bitmaps am Ende der Datei

#define intervall    500                 // Intervallzeit zwischen den Messungen
#define baud         115200

/* ----------------------------------------------------------
                        my_putchar

     wird von my_printf aufgerufen. Hierauf gibt my_printf
     den Datenstream aus
   ---------------------------------------------------------- */
void my_putchar(char ch)
{
  uart_putchar(ch);
}


/* ----------------------------------------------------------------------
                                     main
   ---------------------------------------------------------------------- */
int main(void)
{
  uint16_t      cnt;
  int           temp;

  SystemInit();

  uart_init(baud);
  ds18b20_reset();

  printf("\n\r --------------------------------------\n\r");
  printf(" DS18B20 Temperatursensor - Demo       \n\r");
  printf("                                       \n\r");
  printf("                                       \r");
  printf(" CH32V003 running at %d MHz\n\r", F_CPU/1000000);
  printf("                                       \r");
  printf(" Protokol: %d 8N1\n\r",baud);
  printf("                                       \n\r");
  printf(" 15.04.2025  R. Seelig                 \n\r");
  printf(" --------------------------------------\n\n\r");

  printfkomma= 1;
  cnt= 0;
  while(1)
  {
    temp= ds18b20_gettemp();
    printf("\n\rMess.: \t%d \t Temp.: %k oC", cnt,temp);
    delay(intervall);
    cnt++;
  }
}

// Bitmapmuster ASCII - Code, beginnend mit 0x20
const uint8_t bmps7asc [96]  =
{
// space   !     "     #     $     %     &     '
   0x00, 0x86, 0x22, 0x7E, 0x6D, 0xD2, 0x46, 0x20,
//   (     )     *     +     ,     -     .      /
   0x29, 0x0B, 0x21, 0x70, 0x10, 0x40, 0x80, 0x52,
//   0     1     2     3     4     5     6     7
   0x3F, 0x06, 0x5B, 0x4F, 0x66, 0x6D, 0x7D, 0x07,
//   8     9     :     ;     <     =     >     ?
   0x7F, 0x6F, 0x09, 0x0D, 0x61, 0x48, 0x43, 0xD3,
 //  @     A     B     C     D     E     F     G
   0x5F, 0x77, 0x7C, 0x39, 0x5E, 0x79, 0x71, 0x3D,
//    H     I     J     K     L     M     N     O
   0x76, 0x30, 0x1E, 0x75, 0x38, 0x15, 0x37, 0x3F,
//   P     Q     R     S     T     U     V     W
   0x73, 0x6B, 0x33, 0x6D, 0x78, 0x3E, 0x3E, 0x2A,
//   X     Y     Z     [     \     ]     ^     _
   0x76, 0x6E, 0x5B, 0x39, 0x64, 0x0F, 0x23, 0x08,
//    `    a     b     c     d     e     f     g
   0x02, 0x5F, 0x7C, 0x58, 0x5E, 0x7B, 0x71, 0x6F,
//   h     i     j     k     l     m     n     o
   0x74, 0x10, 0x0C, 0x75, 0x30, 0x14, 0x54, 0x5C,
//  p     q     r     s     t     u     v     w
   0x73, 0x67, 0x50, 0x6D, 0x78, 0x1C, 0x1C, 0x14,
//  x     y     z     {     |     }     ~    blk
   0x76, 0x6E, 0x5B, 0x46, 0x30, 0x70, 0x01, 0x00
};
