/* -----------------------------------------------
                  ch32v003_gpio_raw.h

     Makros und Defines zum einfachen Umgang mit
     GPIO - Portpins

     24.03.2025  R. Seelig
   ----------------------------------------------- */

/*
            +-----------+
  PC1 / sda |1    C   16| PC0
  PC2 / scl |2    H   15| Vdd
        PC3 |3    3   14| gnd
        PC4 |4    2   13| PA2 / osc0 / a0
 PC6 / mosi |5    V   12| PA1 / osc1 / a1
 PC7 / miso |6    0   11| nrst / PD7
 PD1 / swio |7    0   10| PD6 / a6 / urx
   PD4 / a7 |8    3    9| PD5 / a5 / utx
            +-----------+
*/


#ifndef in_ch32v003_gpio
  #define in_ch32v003_gpio

  /* ----------------------------------------------------------------------------------------------
  // moegliche Werte fuer GPIO_SPEEDx sind:
  //   - GPIO_Speed_10MHz
  //   - GPIO_Speed_2MHz
  //   - GPIO_Speed_50MHz
  //   - GPIO_Speed_30MHz

  // Hinweis: scheinbar gibt es im Framework ch32v003fun einen Fehler in der Deklaration und die
  //          Angaben von GPIO_Speed_50MHz und GPIO_Speed_30MHz haben dieselben Auswirkungen.
  //          30MHz waeren wohl richtig, da es im Reference Manual (v1.7) keinen 50MHz-Mode gib
     ---------------------------------------------------------------------------------------------- */

  #define GPIO_SPEEDA                GPIO_Speed_10MHz
  #define GPIO_SPEEDC                GPIO_Speed_10MHz
  #define GPIO_SPEEDD                GPIO_Speed_10MHz

  /* -----------------------------------------------------
                     Setup fuer Init Outputs
     ----------------------------------------------------- */
  #define SET_OUTPUT_MODE_A(pinnr)   { GPIOA->CFGLR &= ~(0x0f << (4*pinnr));     \
                                       GPIOA->CFGLR |= ((GPIO_SPEEDA | GPIO_CNF_OUT_PP) << (4*pinnr)); }

  #define SET_OUTPUT_MODE_C(pinnr)   { GPIOC->CFGLR &= ~(0x0f << (4*pinnr));     \
                                       GPIOC->CFGLR |= ((GPIO_SPEEDC | GPIO_CNF_OUT_PP) << (4*pinnr)); }

  #define SET_OUTPUT_MODE_D(pinnr)   { GPIOD->CFGLR &= ~(0x0f << (4*pinnr));     \
                                       GPIOD->CFGLR |= ((GPIO_SPEEDD | GPIO_CNF_OUT_PP) << (4*pinnr)); }

  /* -----------------------------------------------------
                     Setup fuer Init Inputs

       Hinweis: GPIO_CNF_PUPD initialisiert sowohl einen
       Pullup- sowie einen Pulldown-Widerstand. Der Zu-
       stand im Ausgangsregister bestimmt, ob ein Pull-
       up- oder Pulldown-Widerstand gesetzt wird. Standard-
       maessig ist der Ausgang 0 und somit ein Pulldown-
       widerstand gesetzt (im Gegensatz bspw. zu einem
       STM32). Bei spaeteren Input_Init-Sequenz wird des-
       halb der Pin standardmaessig auf 1 gesetzt.

       Auswahlmoeglichkeit fuer einen Eingangspin ist:

          - GPIO_CNF_IN_PUPD
          - GPIO_CNF_IN_FLOATING
     ----------------------------------------------------- */

  #define SET_INPUT_MODE_A(pinnr)   { GPIOA->CFGLR &= ~(0x0f << (4*pinnr));     \
                                      GPIOA->CFGLR |= ((GPIO_CNF_IN_PUPD) << (4*pinnr)); }

  #define SET_INPUT_MODE_C(pinnr)   { GPIOC->CFGLR &= ~(0x0f << (4*pinnr));     \
                                      GPIOC->CFGLR |= ((GPIO_CNF_IN_PUPD) << (4*pinnr)); }

  #define SET_INPUT_MODE_D(pinnr)   { GPIOD->CFGLR &= ~(0x0f << (4*pinnr));     \
                                      GPIOD->CFGLR |= ((GPIO_CNF_IN_PUPD) << (4*pinnr)); }


  #define setmask(nr)               (1 << nr)
  #define clrmask(nr)               (1 << (16+nr))

  /* -----------------------------------------------------
                         PortA - Outputs
     ----------------------------------------------------- */

  #define PA1_output_init()         SET_OUTPUT_MODE_A(1)
  #define PA1_set()                 GPIOA->BSHR |= setmask(1);
  #define PA1_clr()                 GPIOA->BSHR |= clrmask(1);

  #define PA2_output_init()         SET_OUTPUT_MODE_A(2)
  #define PA2_set()                 GPIOA->BSHR |= setmask(2);
  #define PA2_clr()                 GPIOA->BSHR |= clrmask(2);

  /* -----------------------------------------------------
                         PortA - Inputs
     ----------------------------------------------------- */
  #define PA1_input_init()          { SET_INPUT_MODE_A(1); PA1_set(); }
  #define is_PA1()                  ((int)((GpioOf(PA1)-> INDR >> ((PA1) & 0x0f)) & 1))

  #define PA2_input_init()          { SET_INPUT_MODE_A(2); PA2_set(); }
  #define is_PA2()                  ((int)((GpioOf(PA2)-> INDR >> ((PA2) & 0x0f)) & 1))

  /* -----------------------------------------------------
                         PortC - Outputs
     ----------------------------------------------------- */

  #define PC0_output_init()         SET_OUTPUT_MODE_C(0)
  #define PC0_set()                 GPIOC->BSHR |= setmask(0);
  #define PC0_clr()                 GPIOC->BSHR |= clrmask(0);

  #define PC1_output_init()         SET_OUTPUT_MODE_C(1)
  #define PC1_set()                 GPIOC->BSHR |= setmask(1);
  #define PC1_clr()                 GPIOC->BSHR |= clrmask(1);

  #define PC2_output_init()         SET_OUTPUT_MODE_C(2)
  #define PC2_set()                 GPIOC->BSHR |= setmask(2);
  #define PC2_clr()                 GPIOC->BSHR |= clrmask(2);

  #define PC3_output_init()         SET_OUTPUT_MODE_C(3)
  #define PC3_set()                 GPIOC->BSHR |= setmask(3);
  #define PC3_clr()                 GPIOC->BSHR |= clrmask(3);

  #define PC4_output_init()         SET_OUTPUT_MODE_C(4)
  #define PC4_set()                 GPIOC->BSHR |= setmask(4);
  #define PC4_clr()                 GPIOC->BSHR |= clrmask(4);

  #define PC5_output_init()         SET_OUTPUT_MODE_C(5)
  #define PC5_set()                 GPIOC->BSHR |= setmask(5);
  #define PC5_clr()                 GPIOC->BSHR |= clrmask(5);

  #define PC6_output_init()         SET_OUTPUT_MODE_C(6)
  #define PC6_set()                 GPIOC->BSHR |= setmask(6);
  #define PC6_clr()                 GPIOC->BSHR |= clrmask(6);

  #define PC7_output_init()         SET_OUTPUT_MODE_C(7)
  #define PC7_set()                 GPIOC->BSHR |= setmask(7);
  #define PC7_clr()                 GPIOC->BSHR |= clrmask(7);


  /* -----------------------------------------------------
                         PortC - Inputs
     ----------------------------------------------------- */
  #define PC0_input_init()          { SET_INPUT_MODE_C(0); PC0_set(); }
  #define is_PC0()                  ((int)((GpioOf(PC0)-> INDR >> ((PC0) & 0x0f)) & 1))

  #define PC1_input_init()          { SET_INPUT_MODE_C(1); PC1_set(); }
  #define is_PC1()                  ((int)((GpioOf(PC1)-> INDR >> ((PC1) & 0x0f)) & 1))

  #define PC2_input_init()          { SET_INPUT_MODE_C(2); PC2_set(); }
  #define is_PC2()                  ((int)((GpioOf(PC2)-> INDR >> ((PC2) & 0x0f)) & 1))

  #define PC3_input_init()          { SET_INPUT_MODE_C(3); PC3_set(); }
  #define is_PC3()                  ((int)((GpioOf(PC3)-> INDR >> ((PC3) & 0x0f)) & 1))

  #define PC4_input_init()          { SET_INPUT_MODE_C(4); PC4_set(); }
  #define is_PC4()                  ((int)((GpioOf(PC4)-> INDR >> ((PC4) & 0x0f)) & 1))

  #define PC5_input_init()          { SET_INPUT_MODE_C(5); PC5_set(); }
  #define is_PC5()                  ((int)((GpioOf(PC5)-> INDR >> ((PC5) & 0x0f)) & 1))

  #define PC6_input_init()          { SET_INPUT_MODE_C(6); PC6_set(); }
  #define is_PC6()                  ((int)((GpioOf(PC6)-> INDR >> ((PC6) & 0x0f)) & 1))

  #define PC7_input_init()          { SET_INPUT_MODE_C(7); PC7_set(); }
  #define is_PC7()                  ((int)((GpioOf(PC7)-> INDR >> ((PC7) & 0x0f)) & 1))

  /* -----------------------------------------------------
                         PortD - Outputs
     ----------------------------------------------------- */

  #define PD0_output_init()         SET_OUTPUT_MODE_D(0)
  #define PD0_set()                 GPIOD->BSHR |= setmask(0);
  #define PD0_clr()                 GPIOD->BSHR |= clrmask(0);

  #define PD1_output_init()         SET_OUTPUT_MODE_D(1)
  #define PD1_set()                 GPIOD->BSHR |= setmask(1);
  #define PD1_clr()                 GPIOD->BSHR |= clrmask(1);

  #define PD2_output_init()         SET_OUTPUT_MODE_D(2)
  #define PD2_set()                 GPIOD->BSHR |= setmask(2);
  #define PD2_clr()                 GPIOD->BSHR |= clrmask(2);

  #define PD3_output_init()         SET_OUTPUT_MODE_D(3)
  #define PD3_set()                 GPIOD->BSHR |= setmask(3);
  #define PD3_clr()                 GPIOD->BSHR |= clrmask(3);

  #define PD4_output_init()         SET_OUTPUT_MODE_D(4)
  #define PD4_set()                 GPIOD->BSHR |= setmask(4);
  #define PD4_clr()                 GPIOD->BSHR |= clrmask(4);

  #define PD5_output_init()         SET_OUTPUT_MODE_D(5)
  #define PD5_set()                 GPIOD->BSHR |= setmask(5);
  #define PD5_clr()                 GPIOD->BSHR |= clrmask(5);

  #define PD6_output_init()         SET_OUTPUT_MODE_D(6)
  #define PD6_set()                 GPIOD->BSHR |= setmask(6);
  #define PD6_clr()                 GPIOD->BSHR |= clrmask(6);

  #define PD7_output_init()         SET_OUTPUT_MODE_D(7)
  #define PD7_set()                 GPIOD->BSHR |= setmask(7);
  #define PD7_clr()                 GPIOD->BSHR |= clrmask(7);


  /* -----------------------------------------------------
                         PortD - Inputs
     ----------------------------------------------------- */
  #define PD0_input_init()          { SET_INPUT_MODE_D(0); PD0_set(); }
  #define is_PD0()                  ((int)((GpioOf(PD0)-> INDR >> ((PD0) & 0x0f)) & 1))

  #define PD1_input_init()          { SET_INPUT_MODE_D(1); PD1_set(); }
  #define is_PD1()                  ((int)((GpioOf(PD1)-> INDR >> ((PD1) & 0x0f)) & 1))

  #define PD2_input_init()          { SET_INPUT_MODE_D(2); PD2_set(); }
  #define is_PD2()                  ((int)((GpioOf(PD2)-> INDR >> ((PD2) & 0x0f)) & 1))

  #define PD3_input_init()          { SET_INPUT_MODE_D(3); PD3_set(); }
  #define is_PD3()                  ((int)((GpioOf(PD3)-> INDR >> ((PD3) & 0x0f)) & 1))

  #define PD4_input_init()          { SET_INPUT_MODE_D(4); PD4_set(); }
  #define is_PD4()                  ((int)((GpioOf(PD4)-> INDR >> ((PD4) & 0x0f)) & 1))

  #define PD5_input_init()          { SET_INPUT_MODE_D(5); PD5_set(); }
  #define is_PD5()                  ((int)((GpioOf(PD5)-> INDR >> ((PD5) & 0x0f)) & 1))

  #define PD6_input_init()          { SET_INPUT_MODE_D(6); PD6_set(); }
  #define is_PD6()                  ((int)((GpioOf(PD6)-> INDR >> ((PD6) & 0x0f)) & 1))

  #define PD7_input_init()          { SET_INPUT_MODE_D(7); PD7_set(); }
  #define is_PD7()                  ((int)((GpioOf(PD7)-> INDR >> ((PD7) & 0x0f)) & 1))

  // -------------------------------------------------


#endif
