/* -------------------------------------------------------
                        ds18b20_single.c

     Sourcecode fuer das Einlesen eines einzelnen
     DS18B20 Temperatursensors

     MCU   :  CH32V003

     one_wire_Anschluss DS18B20 auf PA1 (aenderbar)

     02.04.2025  R. Seelig
   ------------------------------------------------------ */

/*
   DS18B20
  +-------+
  |       |
  |       |
  |       |
  +-------+
   |  |  |
   |  |  |
   |  |  |

   G  D  V
   N  A  c
   D  T  c
*/

#include <ds18b20_single.h>

/* -----------------------------------------------------------------------
                               ds18b20_reset

     initialisiert (und resetet somit) den Sensor
   -----------------------------------------------------------------------*/
uint8_t ds18b20_reset()
{
  uint8_t i;

  onewire_clr();
  delay_us(480);

  onewire_set();
  delay_us(60);

  // Bus lesen und warten
  i = onewire_readb();
  delay_us(420);

  return i;               // 0: Bus ok, 1: Fehler
}

/* -----------------------------------------------------------------------
                               ds18b20_writebit

     schreibt ein einzelnes Bit auf dem Bus
   -----------------------------------------------------------------------*/
void ds18b20_writebit(uint8_t bit)
{
  onewire_clr();
  delay_us(1);

  if(bit) onewire_set();
  delay_us(60);                    // 0-1 Bit fuer 60us stehen lassen

  onewire_set();                    // Funktion mit 1 auf Busleitung beenden
}

/* -----------------------------------------------------------------------
                               ds18b20_readbit

     ein Bit vom Bus lesen
   -----------------------------------------------------------------------*/
uint8_t ds18b20_readbit(void)
{
  uint8_t bit=0;

  onewire_clr();
  delay_us(1);

  onewire_set();
  delay_us(14);

  if(onewire_readb()) bit=1;

  delay_us(45);

  return bit;
}

/* -----------------------------------------------------------------------
                               ds18b20_writebyte

     schreibt ein Byte auf dem Bus
   -----------------------------------------------------------------------*/
void ds18b20_writebyte(uint8_t data)
{
  uint8_t i= 8;

  while(i--)
  {
    ds18b20_writebit(data & 1);
    data = data >> 1;
  }
}

/* -----------------------------------------------------------------------
                               ds18b20_readbyte

     schreibt ein Byte auf dem Bus
   -----------------------------------------------------------------------*/
uint8_t ds18b20_readbyte(void)
{
  uint8_t i= 8, data= 0;

  while(i--)
  {
    data = data >> 1;
    data |= (ds18b20_readbit() << 7);
  }
  return data;
}

/* -----------------------------------------------------------------------
                               ds18b20_gettemp

     startet Messvorgang und gibt die gelesene Temperatur als Pseudo-
     kommazahl zurueck.

     Bsp.:  271 entspricht 27,1 oC
   -----------------------------------------------------------------------*/
int16_t ds18b20_gettemp(void)
{
  uint8_t temp_lo_by;
  uint8_t temp_hi_by;

  #if (DS18B20_STOPINTERRUPTONREAD == 1)
    // todo
  #endif

  ds18b20_reset();
  ds18b20_writebyte(0xcc);                      // cmd: skip ROM
  ds18b20_writebyte(0x44);                      // cmd: Messvorgang starten

  while(!ds18b20_readbit());                    // warten bis Messvorgang abgeschlossen

  ds18b20_reset();                              // reset
  ds18b20_writebyte(0xcc);                      // cmd: skip ROM
  ds18b20_writebyte(0xbe);                      // cmd: read scratchpad

  // 2 Bytes aus Scratchpad lesen (12 relevante Bits)
  temp_lo_by = ds18b20_readbyte();
  temp_hi_by = ds18b20_readbyte();

  #if (DS18B20_STOPINTERRUPTONREAD == 1)
    // todo
  #endif

  // setzt die beiden 8 Bit Werte zusammen und gibt die Temperatur als
  // Pseudokommazahl zurueck
  return  ( ( ( temp_hi_by << 8 ) + temp_lo_by ) * 10) / 16;
}
