/* ----------------------------------------------------------
     smallio.c

     Sehr schlankes I/O Interface ueber die
     serielle Schnittstelle mit abgespecktem printf. Hier-
     durch ist es moeglich, die Beispiele der Hilfedatei
     fuer viele Mikrocontroller und auf PC nachzuvollziehen.

     Fuer die unterstuetzten Microcontroller (AVR, STM32,
     MCS-51, STM8, ch32v003) ist jeweils ein eigenes smallio
     verfuegbar, sodass JEDES Programm lediglich im Header
     ein

                  "include smallio.h"

     eingefuegt werden muss. Im Makefile ist einzustellen,
     dass smallio.o fuer AVR,ARM und PC-Konsolenprogramme,
     smallio.rel fuer  MCS-51 und STM8 oder smallio.c fuer
     ch32v003 hinzugelinkt werden muss.

     Eintrag im Makefile:

     SRCS      += ../src/smallio.o

     Die eingestellte Baudrate betraegt bei allen
     Controllern 19200 bd (Ausnahme MCS-51, hier dann
     4800 bd)

     Protokoll ist 8N1


     2019 / 28.05.2025  R. Seelig
     ------------------------------------------------------ */

#include "smallio.h"


char printfkomma = 2;

/* ----------------------------------------------------------
                           smallio_init

     initialisiert serielle Schnittstelle mit Protokoll 8N1
     TxD an Pin PD5, RxD an Pin PD6
   ---------------------------------------------------------- */
void smallio_init()
{
  SystemInit();              // 48 MHz Core-Takt, alle GPIO's an

  // Enable UART and GPIOD
  RCC->APB2PCENR |= RCC_APB2Periph_GPIOD | RCC_APB2Periph_USART1;

  // Push-Pull, 10MHz Output on D5, alternate function (TxD)
  GPIOD->CFGLR = (GPIOD->CFGLR & ~(0x0f << (4*5))) |
                ((GPIO_Speed_10MHz | GPIO_CNF_OUT_PP_AF)<<( 4*5 ));

  // Setup UART for 8n1
  USART1->CTLR1 = USART_WordLength_8b | USART_Parity_No | USART_Mode_Tx | USART_Mode_Rx;
  USART1->CTLR2 = USART_StopBits_1;

  // Set baud rate and enable UART
  USART1->BRR = ((FUNCONF_SYSTEM_CORE_CLOCK) + (baudrate)/2) / (baudrate);
  USART1->CTLR1 |= CTLR1_UE_Set;
}

/* ----------------------------------------------------------
                        putchar

     sendet ein Zeichen auf der seriellen Schnittstelle

     Uebergabe:
       ch : zu sendendes Zeichen
   ---------------------------------------------------------- */
int putchar(int ch)
{
   while(!(USART1->STATR & USART_FLAG_TXE));            // warten bis evtl. vorangegangenes Zeichen gesendet ist
   USART1->DATAR = ch;
   return 0;
}

/* ----------------------------------------------------------
                        keypressed

     testet, ob auf der seriellen Schnittstelle ein Zeichen
     eingegangen ist, liest dieses aber nicht ein

     Rueckgabe:
       1 : es ist ein Zeichen eingetroffen
       0 : kein Zeichen verfuegbar
   ---------------------------------------------------------- */
uint8_t keypressed(void)
{
  return (USART1->STATR & USART_FLAG_RXNE);
}

/* ----------------------------------------------------------
                        getchar

     wartet, bis auf der seriellen Schnittstelle ein Zeichen
     eingegangen ist und liest dieses ein.

     Rueckgabe:
       gelesenes Zeichen
   ---------------------------------------------------------- */
int getchar(void)
{
  while(!(USART1->STATR & USART_FLAG_RXNE));
  return (USART1->DATAR) & 0x00ff;
}


/* ##########################################################
                    Ende Hardware UART
   ########################################################## */


/* ------------------------------------------------------------
                            readint

     liest einen 16-Bit signed Integer auf der seriellen
     Schnittstelle ein (allerdings reicht der Eingabebereich
     nur von -32767 .. +32767).

     Korrektur ist mit der Loeschtaste nach links moeglich.
   ------------------------------------------------------------ */
int16_t readint(void)
{
  uint8_t   signflag= 0;
  uint16_t  sum = 0;
  uint8_t   zif;
  uint8_t   ch;

  do
  {
    ch= getchar();
    if (ch== 0x0a) ch= 0x0d;

    // Ziffern auswerten
    if ((ch>= '0') && (ch<= '9'))
    {
      zif= ch-'0';

      if ((sum== 0) && (zif))        // erste Ziffer
      {
        sum= zif;
        putchar(ch);
      }
      else
      if (sum < 3277)
      {
        if  (!( ((sum * 10) > 32750) && (zif > 7) ))
        {
          sum= (sum*10) + zif;
          putchar(ch);
        }
      }
    }

    // letzte Eingabe loeschen
    if ((ch== 127) || (ch== 8))    // letzte Eingabe loeschen
    {
      if (sum)
      {
        sum /= 10;
        putchar(8);
        putchar(' ');
        putchar(8);
      }
      else
      if (signflag)
      {
        putchar(8);
        putchar(' ');
        putchar(8);
        signflag= 0;
      }
    }

    // Eingabe Minuszeichen
    if ((ch== '-') && (sum == 0))
    {
      signflag= 1;
        putchar('-');
    }

  } while (ch != 0x0d);              // wiederholen bis Returnzeichen eintrifft
  if (signflag) return sum *(-1); else return sum;
}

/* ------------------------------------------------------------
     charout

     sendet ein Zeichen in Abhaengigkeit von Outstream ent-
     weder an das Ausgabegeraet oder in ein Chararray
   ------------------------------------------------------------ */

void charout(char ch)
{
  putchar(ch);
}

/* ------------------------------------------------------------
     putint

     gibt einen Integer dezimal aus. Ist Uebergabe
     "komma" != 0 wird ein "Kommapunkt" mit ausgegeben.
     Groesste darstellare Zahl ist 99.999.999

     Bsp.: 12345 wird als 123.45 ausgegeben.
     (ermoeglicht Pseudofloatausgaben im Bereich)
   ------------------------------------------------------------ */
void putint(int i, char komma)
{
  typedef enum boolean { FALSE, TRUE }bool_t;

  static int zz[]  = { 10000000, 1000000, 100000, 10000, 1000, 100, 10 };
  bool_t not_first = FALSE;

  uint8_t zi;

  komma= 8-komma;

  if (!i)
  {
    charout('0');
  }
  else
  {
    if(i < 0)
    {
      charout('-');
      i = -i;
    }

    int z, b;

    for(zi = 0; zi < 7; zi++)
    {
      z = 0;
      b = 0;

      while(z + zz[zi] <= i)
      {
        b++;
        z += zz[zi];
      }

      if(b || not_first)
      {
        charout('0' + b);
        not_first = TRUE;
      }

      if (zi+1 == komma)
      {
        if (!not_first) charout('0');
        charout('.');
        not_first= TRUE;
      }

      i -= z;
    }
    charout('0' + i);
  }
}

/* ------------------------------------------------------------
     hexnibbleout

     gibt die unteren 4 Bits eines chars als Hexaziffer aus.
     Eine Pruefung ob die oberen vier Bits geloescht sind
     erfolgt NICHT !
  -------------------------------------------------------------  */
void hexnibbleout(uint8_t b)
{
  if (b< 10) b+= '0'; else b+= 55;
  charout(b);
}

/* ------------------------------------------------------------
     puthex

     gibt einen Integer hexadezimal aus. Ist die auszugebende
     Zahl >= 0xff erfolgt die Ausgabe 2-stellig, ist sie
     groesser erfolgt die Ausgabe 4-stellig.

     Ist out16 gesetzt, erfolgt ausgabe immer 4 stellig
   ------------------------------------------------------------ */
void puthex(uint16_t h, char out16)
{
  uint8_t b;

  if ((h> 0xff) || out16)                    // 16 Bit-Wert
  {
    b= (h >> 12);
    hexnibbleout(b);
    b= (h >> 8) & 0x0f;
    hexnibbleout(b);
  }
  b= h;
  b= (h >> 4) & 0x0f;
  hexnibbleout(b);
  b= h & 0x0f;
  hexnibbleout(b);
}


/* ------------------------------------------------------------
     putstring

     gibt einen AsciiZ String aus
  -------------------------------------------------------------  */
void putstring(char *p)
{
  while(*p)
  {
    charout(*p);
    p++;
  }
}

/* ------------------------------------------------------------
     my_printf

     alternativer Ersatz fuer printf.

     Aufruf:

         my_printf("Ergebnis= %d",zahl);

     Platzhalterfunktionen:

        %s     : Ausgabe Textstring
        %d     : dezimale Ausgabe
        %x     : hexadezimale Ausgabe
                 ist Wert > 0xff erfolgt 4-stellige
                 Ausgabe
                 is Wert <= 0xff erfolgt 2-stellige
                 Ausgabe
        %k     : Integerausgabe als Pseudokommazahl
                 12345 wird als 123.45 ausgegeben
        %c     : Ausgabe als Asciizeichen

   ------------------------------------------------------------ */

void my_printf(const char *s,...)
{
  int       arg1;
  uint32_t  xarg1;
  char     *arg2;
  char      ch;
  va_list   ap;

  va_start(ap,s);
  do
  {
    ch= *s;
    if(ch== 0) return;

    if(ch=='%')            // Platzhalterzeichen
    {
      s++;
      uint8_t token= *s;
      switch(token)
      {
        case 'd':          // dezimale Ausgabe
        {
          arg1= va_arg(ap,int);
          putint(arg1,0);
          break;
        }
        case 'x':          // hexadezimale Ausgabe
        {
          xarg1= va_arg(ap,uint32_t);
          if (xarg1 <= 0xFFFF)
          {
            puthex(xarg1, 0);
          }
          else
          {
            puthex(xarg1 >> 16,0);
            puthex(xarg1 & 0xffff,1);
          }

          break;
        }
        case 'k':
        {
          arg1= va_arg(ap,int);
          putint(arg1,printfkomma);     // Integerausgabe mit Komma: 12896 zeigt 12.896 an
          break;
        }
        case 'c':          // Zeichenausgabe
        {
          arg1= va_arg(ap,int);
          charout(arg1);
          break;
        }
	case '%':
        {
          charout(token);
          break;
        }
        case 's':
        {
          arg2= va_arg(ap,char *);
          putstring(arg2);
          break;
        }
      }
    }
    else
    {
      charout(ch);
    }
    s++;
  }while (ch != '\0');
}

