/* --------------------------------------------------------
                         ws2812_demo.c

     MCU : ch32v003

     14.04.2025  R. Seelig
   -------------------------------------------------------- */

/*

                  CH32V003 A4M6                                        CH32V003F4P6

                  +-----------+                                      +--------------+
        PC1 / sda |1    C   16| PC0                         PD4 / a7 | 1         20 | PD3 / a4
        PC2 / scl |2    H   15| Vdd                   PD5 / a5 / utx | 2    C    19 | PD2 / a3
              PC3 |3    3   14| gnd                   PD6 / a6 / urx | 3    H    18 | PD1 / swio
              PC4 |4    2   13| PA2 / osc0 / a0           nrst / PD7 | 4    3    17 | PC7 / miso
       PC6 / mosi |5    V   12| PA1 / osc1 / a1      PA1 / osc0 / a0 | 5    2    16 | PC6 / mosi
       PC7 / miso |6    0   11| nrst / PD7           PA2 / osc1 / a1 | 6    V    15 | PC5 / sck / scl
       PD1 / swio |7    0   10| PD6 / a6 / urx                   gnd | 7    0    14 | PC4 / a2
         PD4 / a7 |8    3    9| PD5 / a5 / utx                   PD0 | 8    0    13 | PC3
                  +-----------+                                  Vdd | 9    3    12 | PC2 / scl
                                                                 PC0 | 10        11 | PC1 / sda
                                                                     +--------------+

*/
#include <string.h>
#include <stdlib.h>

#include "ch32fun.h"
#include "ch32v003_gpio.h"

#define ws2812_port     GPIOD              // PD4 ist ws2812-data Anschluss
#define ws2812_out      4

#define ws2812_clr()    (ws2812_port->BCR  = 1 << ws2812_out)
#define ws2812_set()    (ws2812_port->BSHR = 1 << ws2812_out)

#define ws2812_init()   { ws2812_port->CFGLR &= ~(0x0f << (4*ws2812_out));     \
                          ws2812_port->CFGLR |= ((1 | GPIO_CNF_OUT_PP) << (4*ws2812_out)); }   // 1 GPIO-Takt 10 MHz
                                                                                               // 2 GPIO-Takt  2 MHz
                                                                                               // 3 GPIO-Takt 30 MHz
                                                                                               // Farbstruktur rot, gruen, blau

typedef struct colvalue
{
  uint8_t r;
  uint8_t g;
  uint8_t b;
} colvalue;


#define ledanz    12            // 12 Leuchtdioden auf dem WS2812-Bus


#if (ledanz < 7)
  uint8_t ledcol[7*3];          // damit die Schiebe-Rotieren Funktionen korrekt funktionieren
                                // toDo in led_shlrol / led_shrror
#else
  uint8_t ledcol[ledanz*3];
#endif

/* ------------------------------------------------------------------------------------
     Farbwerte wie sie einer uralten EGA-Grafikkarte mit 16
     Farben enstpricht:

        0       1     2       3      4      5       6      7
     schwarz, blau, gruen, tuerkis, rot, violett, braun, grau

        8            9        10          11          12        13        14    15
     dunkelgrau, hellblau, hellgruen, helltuerkis, hellrot, hellviolett, gelb, weiss


     Farbabstufung mit hellerer Leuchtstaerke:

        16     17     18     19      20    21       22    23
     schwarz, blau, gruen, tuerkis, rot, violett, braun, grau

        24          25        26          27          28        29        30    31
     dunkelgrau, hellblau, hellgruen, helltuerkis, hellrot, hellviolett, gelb, weiss
   ----------------------------------------------------------------------------------- */
const uint8_t egapalette[] =
{
/* -------------------------------------------------------------
      Farbpalette fuer dunklere EGA Farben
   ------------------------------------------------------------- */
  0x00,0x00,0x00, 0x00,0x00,0x03, 0x00,0x02,0x00, 0x00,0x03,0x03,       // 0..3
  0x03,0x00,0x00, 0x03,0x00,0x03, 0x03,0x01,0x00, 0x07,0x07,0x07,       // 4..7
  0x01,0x01,0x01, 0x01,0x01,0x06, 0x00,0x09,0x00, 0x01,0x06,0x06,       // 8..11
  0x08,0x00,0x00, 0x06,0x01,0x06, 0x07,0x02,0x00, 0x08,0x08,0x08,       // 11..15
/* -------------------------------------------------------------
      Farbpalette fuer helle EGA Farben
   ------------------------------------------------------------- */
  0x00,0x00,0x00, 0x00,0x00,0xaa, 0x00,0xaa,0x00, 0x00,0xaa,0xaa,       // 16..19
  0xaa,0x00,0x00, 0xaa,0x00,0xaa, 0xaa,0x55,0x00, 0xaa,0xaa,0xaa,       // 20..23
  0x55,0x55,0x55, 0x55,0x55,0xff, 0x05,0xff,0x05, 0x05,0xff,0xff,       // 24..27
  0xff,0x00,0x00, 0xbb,0x00,0xff, 0xff,0x60,0x00, 0xff,0xff,0xff        // 28..31
};

/* ----------------------------------------------------------
                            rgbfromega

     beschreibt eine struct Variable auf die *f zeigt mit
     einem Farbwert, der aus der EGA-Palette entnommen wird

     Uebergabe:
                     nr   : Farbeintrag der EGA-Palette

     Rueckgabe:
                     rgb-struct mit dem Farbinhalt der ge-
                     waehlten EGA-Palette
   ---------------------------------------------------------- */
colvalue rgbfromega(uint8_t eganr)
{
  colvalue f;

  f.g= egapalette[(eganr*3)+1];     // gruen
  f.r= egapalette[(eganr*3)];       // rot
  f.b= egapalette[(eganr*3)+2];     // blau
  return f;
}

/* ----------------------------------------------------------
                          rgbfromvalue

     beschreibt eine struct Variable auf die *f zeigt mit
     einem RGB Farbwert

     Uebergabe:
                  r,g,b   : RGB - Farbwert

     Rueckgabe:
                     rgb-struct mit dem Farbinhalt der ge-
                     setzten RGB-Farbwerte
   ---------------------------------------------------------- */
colvalue rgbfromvalue(uint8_t r, uint8_t g, uint8_t b)
{
  colvalue f;

  f.r= r;
  f.g= g;
  f.b= b;
  return f;
}

/* ----------------------------------------------------------
                            ws2812_send

     beschreibt eine struct Variable auf die *f zeigt mit
     einem Farbwert, der aus der EGA-Palette entnommen wird

     Uebergabe:
         *data : Zeiger auf einen Pufferspeicher, der die
                 Farbwerte der LEDs beinhaltet. Dieser be-
                 inhaltet jeweils 3 Bytes fuer eine LED in
                 der Reihenfolge gruen, rot, blau.

       Hinweis : Beim Belegen eines LED-Arrays mittels
                 led_setrgbcol erwartet diese die Farben-
                 folge rot-gruen-blau

            anz: Anzahl der zu sendenden Farbwerte. Hier
                 werden je 3 Byte per anz verschickt.
                 Anzugeben sind hier die Anzahl der LEDs
                 und nicht die zu sendenden Bytes
   ---------------------------------------------------------- */
void ws2812_send(uint8_t *data, uint8_t anz)
{
  uint8_t mask;
  uint8_t col;
  uint8_t n;

  int8_t ind[] = { 0, 0, 0 };         // Farbreihenfolge der WS2812: 0,  0,  0 => gruen, rot, blau
                                      //                             1, -1,  0 => rot, gruen, blau
                                      //                             2,  0, -1 => blau, gruen, rot

  anz= anz*3;
  for (n = 0; n < anz; n++)
  {
    mask= 0x80;
    col = data[n+ind[n % 3]];

    while (mask)                             // 24 Bit
    {
      // Pulszeit
      ws2812_set();
      __asm volatile(" nop ");
      __asm volatile(" nop ");
      __asm volatile(" nop ");
      __asm volatile(" nop ");
      __asm volatile(" nop ");
      __asm volatile(" nop ");
      __asm volatile(" nop ");
      __asm volatile(" nop ");
      __asm volatile(" nop ");
      if (!(col & mask))                  // soll 0 gesendet werden => Pin nach Lo
                                          // ansonsten Pin auf 1 lassen
      {
        // lange Pausezeit wenn Lo gesendet wird
        ws2812_clr();
      }
      __asm volatile(" nop ");
      __asm volatile(" nop ");
      __asm volatile(" nop ");
      __asm volatile(" nop ");
      __asm volatile(" nop ");
      __asm volatile(" nop ");
      __asm volatile(" nop ");
      __asm volatile(" nop ");
      __asm volatile(" nop ");
      __asm volatile(" nop ");
      __asm volatile(" nop ");
      __asm volatile(" nop ");
      __asm volatile(" nop ");
      __asm volatile(" nop ");

      ws2812_clr();

      if ((col & mask))                   // Ausgleich fuer GPIO-Pinwackeln
      {   __asm volatile(" nop ");
          __asm volatile(" nop ");
          __asm volatile(" nop ");
      }
      __asm volatile(" nop ");
      __asm volatile(" nop ");
      mask = mask >> 1;                          // naechstes Bit
   }
 }
}

/* ----------------------------------------------------------
                            ws2812_clearlall

     schaltet Anzahl anz auf dem WS2812 LED-Bus aus (nicht
     im LED-Array)
   ---------------------------------------------------------- */
void ws2812_clearall(uint8_t anz)
{
  int      i;
  uint8_t  x[3] = {0,0,0};

  for (i= 0; i< (anz*3); i++)
  {
    ws2812_send(&x[0], 1);
  }
  delay(2);
}

/* ----------------------------------------------------------
                        led_setrgbcol

      setzt den Farbwert der in

                    struct colvalue f

      angegeben ist, in ein Array an Stelle nr ein. Dieses
      Array kann mittels ws_showbuffer auf einen LED-Strang
      ausgegeben werden kann.

      Parameter:
                    *ptr : Zeiger auf ein Array, das die
                           RGB Farbwerte aufnehmen soll
                      nr : LED-Nummer im Array, welche die
                           Position im Leuchtdiodenstrang
                           repraesentiert
      struct colvalue *f : Zeiger auf einen RGB-Farbwert

      Usage:
                      rgbfromvalue(50,0,50, &rgbcol);
                      led_setrgbcol(&ledbuffer[0], 4, &rgbcol);

   ---------------------------------------------------------- */
void led_setrgbcol(uint8_t *ptr, uint16_t nr, struct colvalue f)
{
  ptr+= (nr*3);
  *ptr= f.g;
  ptr++;
  *ptr= f.r;
  ptr++;
  *ptr= f.b;
}

/* ----------------------------------------------------------
                          led_clrarray

     loescht ein LED Anzeigearray

     Parameter:
                   *ptr : Zeiger auf ein Array, das
                          geloescht werden soll
                    anz : Anzahl der LEDs, die angezeigt
                          werden sollen
   ---------------------------------------------------------- */
void led_clrarray(uint8_t *ptr, int anz)
{
  memset(ptr, 0x00, anz*3);
}

void led_fillarray(uint8_t *ptr, int anz, struct colvalue f)
{
  int i;

  for (i= 0; i< anz; i++)
  {
    led_setrgbcol(ptr, i, f);
  }
}

/* ----------------------------------------------------------
                        led_shlrol

                  led shift rotate left

     rotiert ein Leuchtdioden-Array um eine Stelle nach
     links. Flag shift_rotate gibt an, ob das Array rotiert
     wird (die 0-te LED wird am Ende eingefuegt),
     oder ob das Array geshiftet wird (die letzte LED wird
     geloescht).

     Uebergabe:
        anz          : Anzahl der LED
        shift_rotate : Flag => 0 shift
                               1 rotate
        *parray      : Zeiger auf das LED-Array, das
                       geshiftet / rotiert wird
   ---------------------------------------------------------- */
void led_shlrol(uint8_t *parray, int anz, uint8_t shift_rotate)
{
  uint8_t *ptr;
  uint8_t *ptr_end;
  uint8_t b0, b1, b2;
  int     i;

  anz= anz*3;
  ptr= parray;
  parray += 3;;
  ptr_end = ptr + anz - 3;

  b0= 0; b1= 0; b2= 0;

  if (shift_rotate== 1)
  {
    // fuer Rollen ersten Farbeintrag (3 Bytes) sichern
    b0= *(ptr);
    b1= *(ptr+1);
    b2= *(ptr+2);
  }

  for (i= 0; i< anz; i++)
  {
    *ptr= *parray;
    parray++;
    ptr++;
  }

  *ptr_end= b0;
  ptr_end++;
  *ptr_end= b1;
  ptr_end++;
  *ptr_end= b2;
}

/* ----------------------------------------------------------
                        led_shrror

                  led shift rotate right

     rotiert ein Leuchtdioden-Array um eine Stelle nach
     rechts. Flag shift_rotate gibt an, ob das Array rotiert
     wird (die letze LED am Ende wird am Anfang eingefuegt),
     oder ob das Array geshiftet wird (die 0-te LED wird
     geloescht).

     Uebergabe:
        anz          : Anzahl der LED
        shift_rotate : Flag => 0 shift
                               1 rotate
        *parray      : Zeiger auf das LED-Array, das
                       geshiftet / rotiert wird
   ---------------------------------------------------------- */
void led_shrror(uint8_t *parray, int anz, uint8_t shift_rotate)
{
  uint8_t *ptr;
  uint8_t *ptr_anf;
  uint8_t b0, b1, b2;
  int     i;

  anz= anz*3;
  ptr = parray;
  ptr_anf = parray;
  parray += anz - 1;

  b0= 0; b1= 0; b2= 0;

  if (shift_rotate== 1)
  {
    // fuer Rollen ersten Farbeintrag (3 Bytes) sichern
    b0= *(ptr+anz-3);
    b1= *(ptr+anz-2);
    b2= *(ptr+anz-1);
  }
  ptr= parray-3;
  for (i= 0; i< anz; i++)
  {
    *parray= *ptr;
    parray--;
    ptr--;
  }

  *ptr_anf= b0;
  ptr_anf++;
  *ptr_anf= b1;
  ptr_anf++;
  *ptr_anf= b2;
}

/* ----------------------------------------------------------
                        ws2812_shrror

                  led-bus shift rotate right

     wie led_shrror mit anschliessender Ausgabe auf dem
     LED-Bus
   ---------------------------------------------------------- */
void ws2812_shrror(uint8_t *parray, int anz, uint8_t shift_rotate)
{
  led_shrror(parray, anz, shift_rotate);
  ws2812_send(parray, anz);
}

/* ----------------------------------------------------------
                        ws2812_shlrol

                  led-bus shift rotate left

     wie led_shlrol mit anschliessender Ausgabe auf dem
     LED-Bus
   ---------------------------------------------------------- */
void ws2812_shlrol(uint8_t *parray, int anz, uint8_t shift_rotate)
{
  led_shlrol(parray, anz, shift_rotate);
  ws2812_send(parray, anz);
}

/* ----------------------------------------------------------
                    ws2812_blendup_right

     blendet nacheinander nach rechts die LEDs auf dem
     LED-Bus mit derselben Farbe auf

     Uebergabe:
        parray : Zeiger auf das LED Array
        anz    : Anzahl LEDs am Bus
        dtime  : Verzoegerungszeit beim Aufblenden
        f      : Farbe der LED
   ---------------------------------------------------------- */
void ws2812_blendup_right(uint8_t *parray, int anz, int dtime, struct colvalue f)
{
  int i,d;

  for (i= 0; i< anz; i++)
  {
    led_setrgbcol(parray, i, f);
    ws2812_send(parray, anz);
    for (d= 0; d< dtime; d++) { delay(1); }
  }
}

/* ----------------------------------------------------------
                    ws2812_blendup_left

     blendet nacheinander nach rechts die LEDs auf dem
     LED-Bus mit derselben Farbe auf

     Uebergabe:
        parray : Zeiger auf das LED Array
        anz    : Anzahl LEDs am Bus
        dtime  : Verzoegerungszeit beim Aufblenden
        f      : Farbe der LED
   ---------------------------------------------------------- */
void ws2812_blendup_left(uint8_t *parray, int anz, int dtime, struct colvalue f)
{
  int i,d;

  for (i= anz; i> -1; i--)
  {
    led_setrgbcol(parray, i, f);
    ws2812_send(parray, anz);
    for (d= 0; d< dtime; d++) { delay(1); }
  }
}


#define speed    50
#define ledbus   &ledcol[0]         // weil ich nicht immer &ledcol[0] schreiben moechte

/* ----------------------------------------------------------
                               main
   ---------------------------------------------------------- */
int main()
{
  uint8_t i, rep;

  SystemInit();

  ws2812_init();

  led_clrarray(ledbus, ledanz);
  ws2812_clearall(ledanz);

  while(1)
  {

    // LEDs auf- und abblenden
    for (rep= 0; rep< 2; rep++)
    {
      ws2812_blendup_right(ledbus, ledanz, speed, rgbfromega(5));
      ws2812_blendup_right(ledbus, ledanz, speed, rgbfromega(0));
    }

    for (rep= 0; rep< 2; rep++)
    {
      ws2812_blendup_left(ledbus, ledanz, speed, rgbfromega(6));
      ws2812_blendup_left(ledbus, ledanz, speed, rgbfromega(0));
    }
    delay(1000);

    // LED-Array mit Farben fuellen
    led_fillarray(ledbus, ledanz, rgbfromvalue(1,0,0));
    led_setrgbcol(ledbus, 5, rgbfromvalue(00,0,30));
    led_setrgbcol(ledbus, 11, rgbfromvalue(30,30,0));

    ws2812_send(ledbus, ledanz);
    delay(speed);

    // und rotieren lassen
    for (rep= 0; rep< 2; rep++)
    {
      for (i= 0; i< 36; i++)
      {
        ws2812_shlrol(ledbus, ledanz, 1);
        delay(speed);
      }
      for (i= 0; i< 36; i++)
      {
        ws2812_shrror(ledbus, ledanz, 1);
        delay(speed);
      }
    }
    delay(2000);
    ws2812_blendup_left(ledbus, ledanz, speed, rgbfromega(0));
    delay(2000);

  }
}
